local UH = UnderHood
local _G = _G
local L = LibStub( "AceLocale-3.0" ):GetLocale( "UnderHood" )

local Provider = {}
Provider.__index = Provider
Provider.name = "Health"
Provider.title = L["Health"]
Provider.supportedUnits = { "player", "target", "focus", "pet", "targettarget" }
	
function Provider:Create( bar )
	local self = {}
		
	setmetatable( self, Provider )
	LibStub( "AceEvent-3.0" ):Embed( self )
	LibStub( "AceTimer-3.0" ):Embed( self )
		
	self.bar = bar
		
	return self
end
	
function Provider:UnitSupported( unit )
	return true
end

function Provider:Connect()
	self:Update()

	if self.bar.config.unit ~= "targettarget" then
		self:RegisterEvent( "UNIT_HEALTH" )
		self:RegisterEvent( "UNIT_MAXHEALTH" )
	end
		
	if self.bar.config.unit == "player" then
		self:RegisterEvent( "PLAYER_ENTERING_WORLD", "Update" )
	elseif self.bar.config.unit == "target" then
		self:RegisterEvent( "PLAYER_TARGET_CHANGED", "Update" )
	elseif self.bar.config.unit == "focus" then
		self:RegisterEvent( "PLAYER_FOCUS_CHANGED", "Update" )
	elseif self.bar.config.unit == "pet" then
	    self:RegisterEvent( "UNIT_PET", "Update" )
	else
	    self:ScheduleRepeatingTimer( "UpdateUneventedUnit", 0.1 )
	end
end

function Provider:Disconnect()
	self:UnregisterAllEvents()
	self:CancelAllTimers()
end

function Provider:UpdateUneventedUnit()
	if not _G.UnitGUID then
	    self:Update()
	    return
	end
	
	local unit = self.bar.config.unit
	local totGuid = UnitGUID( unit )

	if self.totGuid ~= totGuid then
	    -- Current ToT unit is not the same as last one = updating
		self.totGuid = totGuid
		self.totHealth = nil
		self:Update()
	elseif self.totGuid then
		local current, max = UnitHealth( unit ), UnitHealthMax( unit )

		current = max > 0 and current / max or 0

		if self.totHealth ~= current then
		    self.totHealth = current
		    self.bar:SetValue( current )
		end
	end
end

function Provider:UNIT_HEALTH( event, unit )
	if unit == self.bar.config.unit then
		self:SetValue()
	end
end
	
function Provider:UNIT_MAXHEALTH( event, unit )
	if unit == self.bar.config.unit then
		self:SetValue()
	end
end
	
function Provider:Update()
	local r, g, b = unpack( UH.db.profile.colors.health.max )
	local unit = self.bar.config.unit
	local _, class = UnitClass( unit )
	local rc = _G.RAID_CLASS_COLORS[class]

	if not UnitExists( unit ) then return end
	
	if self.bar.config.providerSettings and self.bar.config.providerSettings.colorByHostility then
		if UnitIsPlayer( unit ) or UnitPlayerControlled( unit ) then
			if UnitCanAttack( unit, "player") then
				-- they can attack me
				if UnitCanAttack( "player", unit ) then
					-- and I can attack them
					r, g, b = unpack( UH.db.profile.colors.unit.hostile )
				else
					-- but I can't attack them
					r, g, b = unpack( UH.db.profile.colors.unit.civilian )
				end
			elseif UnitCanAttack( "player", unit ) then
				-- they can't attack me, but I can attack them
				r, g, b = unpack( UH.db.profile.colors.unit.neutral )
			elseif UnitIsPVP( unit ) then
				-- on my team
				if self.bar.config.providerSettings.colorByClass and rc then
				    r, g, b = rc.r, rc.g, rc.b
				else
					r, g, b = unpack( UH.db.profile.colors.unit.friendly )
				end
			else
				-- either enemy or friend, no violence
				r, g, b = unpack( UH.db.profile.colors.unit.civilian )
			end
		elseif (UnitIsTapped( unit ) and not UnitIsTappedByPlayer( unit )) or UnitIsDead( unit ) then
			r, g, b = unpack( UH.db.profile.colors.unit.tapped )
		else
			local reaction = UnitReaction( unit, "player" )
			if reaction then
				if reaction >= 5 then
					if self.bar.config.providerSettings.colorByClass and rc then
					    r, g, b = rc.r, rc.g, rc.b
					else
						r, g, b = unpack( UH.db.profile.colors.unit.friendly )
					end
				elseif reaction == 4 then
					r, g, b = unpack( UH.db.profile.colors.unit.neutral )
				else
					r, g, b = unpack( UH.db.profile.colors.unit.hostile )
				end
			else
				r, g, b = unpack( UH.db.profile.colors.unit.unknown )
			end
		end
	elseif self.bar.config.providerSettings and self.bar.config.providerSettings.colorByClass then
		if rc then
			r, g, b = rc.r, rc.g, rc.b
		else
			r, g, b = unpack( UH.db.profile.colors.unit.unknown )
		end
	end
	
	self.bar:SetColor( r, g, b, 1 )

	self:SetValue()
end
	
function Provider:SetValue()
	local unit = self.bar.config.unit
	local current, max = UnitHealth( unit ), UnitHealthMax( unit )
		
	if max > 0 then
		current = current / max
	else
		current = 0
	end
		
	self.bar:SetValue( current )
end

function Provider:DefaultSettings( config )
	config.providerSettings = {}
end

function Provider:SetupOptions( options )
	options.colorByHostility = {
		name = L["Color by hostility"],
		type = "toggle",
		get = function() return self.bar.config.providerSettings and self.bar.config.providerSettings.colorByHostility end,
		set = function( info, value )
			if not self.bar.config.providerSettings then
				self.bar.config.providerSettings = {}
			end
			
			self.bar.config.providerSettings.colorByHostility = value
			self:Update()
		end,
	}
	
	options.colorByClass = {
		name = L["Color by class"],
		type = "toggle",
		get = function() return self.bar.config.providerSettings and self.bar.config.providerSettings.colorByClass end,
		set = function( info, value )
			if not self.bar.config.providerSettings then
				self.bar.config.providerSettings = {}
			end
			
			self.bar.config.providerSettings.colorByClass = value
			self:Update()
		end,
	}
end
	
UnderHood_Bars:RegisterProvider( Provider )
